'''
For consistency of Setu API across languages, all setu requester API follows Java coding convention in its interface.
This should also be adhered for any public classes and their public methods in a module internally used.

The structure of imports should be kept almost same so that users have little cognitive overhead.
'''