'''
Dummy Remote implementation for testability purpose.

Depending on dummy area, it could be a fully/partially working dummy actor.

From testability aspect, it should assert whether expected packet was got. TO DO!!!
'''